;| Structures |;
Structure ROOM_TILE
	iFloor.i
	fLevel.f
	fHeight.f
	iCeiling.i
	iWall.i[4]
EndStructure

Structure ROOM_WINDOW Extends UBE_VECTOR2f
	fRotation.f
EndStructure

Structure ROOM_OBJECT
	sName.s
	iMesh.i
	iObject.i
	List lLight.i()
	List lParticle.i()
EndStructure

Structure ROOM_DOOR Extends UBE_VECTOR3f
	bStatus.i
	Time.i
	v3fOffset.UBE_VECTOR3f
	iObject.i
	fRotation.f
	List lLight.i()
	List lParticle.i()
EndStructure


Structure ROOM
	iID.i
	bStatus.i
	Size.UBE_SIZE3i
	Position.UBE_VECTOR3f
	iMesh.i
	iObject.i
	List lObject.ROOM_OBJECT()
	List lLight.i()
	List lDoor.ROOM_DOOR()
	List lParticle.i()
	List *plRoom.ROOM()
	Array aTileMap.ROOM_TILE(0)
	Array aDataMap.i(0)
EndStructure

Structure ROOMS
	
	*pFloorMaterial.UBE_MATERIAL
	*pFloorMesh.UBE_MESH
	*pCeilingMaterial.UBE_MATERIAL
	*pCeilingMesh.UBE_MESH
	*pWallMaterial.UBE_MATERIAL
	*pWallMesh.UBE_MESH
	*pWindowMaterial.UBE_MATERIAL
	*pWindowMesh.UBE_MESH
	*pWindowGlassMaterial.UBE_MATERIAL
	*pWindowGlassMesh.UBE_MESH
	*pDoorMaterial.UBE_MATERIAL
	*pDoorMesh.UBE_MESH
	
	List   lRoom.ROOM()
	Array *paRoom.ROOM(0)
EndStructure


;| Variables |;
Global Rooms.ROOMS


;| Loadings |;
UBE_LoadDefinition("Data\Definitions\Room.ubd")
	UBE_LoadAllTextures() 
	UBE_LoadAllMaterials() 
UBE_CloseDefinition()

Rooms\pFloorMesh 						= UBE_LoadOBJMesh(#UBE_ANY, "Data\Models\Floor.obj")
															UBE_CalculateMeshTangents(Rooms\pFloorMesh)	
Rooms\pFloorMaterial				= UBE_Material("Floor")
									 				
Rooms\pCeilingMesh 					= UBE_LoadOBJMesh(#UBE_ANY, "Data\Models\Ceiling.obj")
															UBE_CalculateMeshTangents(Rooms\pCeilingMesh)										
Rooms\pCeilingMaterial 			= UBE_Material("Ceiling")										
												
Rooms\pWallMesh 						= UBE_LoadOBJMesh(#UBE_ANY, "Data\Models\Wall.obj")
									 						UBE_CalculateMeshTangents(Rooms\pWallMesh)										 
Rooms\pWallMaterial 				= UBE_Material("Wall")											 				
									 				
Rooms\pWindowMesh						= UBE_LoadOBJMesh(#UBE_ANY, "Data\Models\Wall Window.obj")
															UBE_CalculateMeshTangents(Rooms\pWindowMesh)	
Rooms\pWindowMaterial 			= UBE_Material("Window")				
									 		
Rooms\pWindowGlassMesh 			= UBE_LoadOBJMesh(#UBE_ANY, "Data\Models\Wall Window Glass.obj")
									 						UBE_CalculateMeshTangents(Rooms\pWindowGlassMesh)											 		
Rooms\pWindowGlassMaterial	= UBE_Material("Window Glass")					 				
									 				
Rooms\pDoorMesh 						= UBE_LoadOBJMesh(#UBE_ANY, "Data\Models\Door.obj")
									 						UBE_CalculateMeshTangents(Rooms\pDoorMesh)												 				
Rooms\pDoorMaterial 				= UBE_Material("Door")										 						


;| Procedures |;							 				
Procedure.i GenerateFloor( iID.i )
	
	Protected Dim TileStatusMap.i(0, 0)
	Protected I.i, X.i, Y.i, XX.i, YY.i
	Protected Size.UBE_SIZE2i
	Protected *Tile.ROOM_TILE
	Protected	*SubTile.ROOM_TILE
	Protected Vertex.i
	Protected Lock.i
	
	With Rooms	 
		
			\paRoom(iID)\iMesh = UBE_CreateMesh(#UBE_ANY)		

			Dim TileStatusMap.i(\paRoom(iID)\Size\iWidth, \paRoom(iID)\Size\iHeight)

			For Y = 0 To \paRoom(iID)\Size\iHeight-1
			For X = 0 To \paRoom(iID)\Size\iWidth-1
				
				If TileStatusMap(X, Y) = #False
					
					*Tile = \paRoom(iID)\aTileMap(X + Y * \paRoom(iID)\Size\iWidth)
					
					If *Tile\iFloor = 1
					
						Size\iWidth 	= 1
						Size\iHeight 	= 1
						Lock 				 = -1

						For YY = Y To \paRoom(iID)\Size\iHeight-1
							For XX = X To \paRoom(iID)\Size\iWidth-1
								
								If X = \paRoom(iID)\Size\iWidth-1
										If YY = \paRoom(iID)\Size\iHeight-1
											Break 2
										EndIf
										
										Lock =  \paRoom(iID)\Size\iWidth
										Break
									EndIf
								
								If XX = X And YY = Y
								XX + 1 
								EndIf
								
								If XX = Lock
									Break 
								EndIf
									
		 						*SubTile = \paRoom(iID)\aTileMap(XX + YY * \paRoom(iID)\Size\iWidth)

		 						If Lock = -1 
		 							If *SubTile\iFloor = *Tile\iFloor And *SubTile\fLevel = *Tile\fLevel And TileStatusMap(XX, YY) = #False 
										Size\iWidth + 1 
									Else
										Lock = XX
										Break
									EndIf
									
								ElseIf Lock > -1
									If *SubTile\iFloor <> *Tile\iFloor Or *SubTile\fLevel <> *Tile\fLevel Or TileStatusMap(XX, YY) = #True
	 									Size\iHeight - 1 
	 									Break 2
	 								EndIf

								EndIf
								
								If XX = \paRoom(iID)\Size\iWidth-1
									Lock = \paRoom(iID)\Size\iWidth
									Break
								EndIf
								
								If YY = \paRoom(iID)\Size\iHeight-1
									Break 2
								EndIf
							Next
							
							Size\iHeight + 1 
						Next
						
						Vertex = UBE_AddMeshData(\paRoom(iID)\iMesh, Rooms\pFloorMesh, X-\paRoom(iID)\Size\iWidth/2, *Tile\fLevel * 2.0, Y-\paRoom(iID)\Size\iHeight/2, 0,0,0, Size\iWidth, 1.0,  Size\iHeight)
							
						For I = 0 To 5
							Protected *TexCoord.UBE_VECTOR2f = UBE_GetMeshTexCoord(\paRoom(iID)\iMesh, Vertex+I)
							*TexCoord\U * Size\iWidth
							*TexCoord\V * Size\iHeight
						Next

						For YY = Y To Y + Size\iHeight-1
						For XX = X To X + Size\iWidth-1
							TileStatusMap(XX, YY) = #True
						Next
						Next

					EndIf
				EndIf
				
				TileStatusMap(X, Y) = #True
				
			Next
			Next

			UBE_FinishMesh(\paRoom(iID)\iMesh)
			UBE_CalculateMeshTangents(\paRoom(iID)\iMesh)	
	
			\lRoom()\iObject 	= UBE_CreateObject(#UBE_ANY, \paRoom(iID)\iMesh, \pFloorMaterial)
													UBE_MoveObject(\paRoom(iID)\iObject, \lRoom()\Position\X, 0, \lRoom()\Position\Z)																 
													UBE_SetObjectBody(\paRoom(iID)\iObject, #UBE_COLLISION_BODY_MESH)

	EndWith
	
EndProcedure

Procedure.i GenerateCeiling( iID.i )
	
	Protected Dim TileStatusMap.i(0, 0)
	Protected I.i, X.i, Y.i, XX.i, YY.i
	Protected Size.UBE_SIZE2i
	Protected *Tile.ROOM_TILE
	Protected	*SubTile.ROOM_TILE
	Protected Vertex.i
	Protected Lock.i
	
	With Rooms	 
		
		AddElement(\paRoom(iID)\lObject())
		\paRoom(iID)\lObject()\sName = "Ceiling" 
		\paRoom(iID)\lObject()\iMesh = UBE_CreateMesh(#UBE_ANY)		

		Dim TileStatusMap.i(\paRoom(iID)\Size\iWidth, \paRoom(iID)\Size\iHeight)

		For Y = 0 To \paRoom(iID)\Size\iHeight-1
		For X = 0 To \paRoom(iID)\Size\iWidth-1
				
			If TileStatusMap(X, Y) = #False
					
					*Tile = \paRoom(iID)\aTileMap(X + Y * \paRoom(iID)\Size\iWidth)
					
					If *Tile\iFloor = 1
					
						Size\iWidth 	= 1
						Size\iHeight 	= 1
						Lock 				 = -1
						
						
							For YY = Y To \paRoom(iID)\Size\iHeight-1
								For XX = X To \paRoom(iID)\Size\iWidth-1
									
									If X = \paRoom(iID)\Size\iWidth-1
										If YY = \paRoom(iID)\Size\iHeight-1
											Break 2
										EndIf
										
										Lock =  \paRoom(iID)\Size\iWidth
										Break
									EndIf
									
									
									
									If XX = X And YY = Y
									XX + 1 
									EndIf
									
									If XX = Lock
										Break 
									EndIf
									
									
			 						
			 						*SubTile = \paRoom(iID)\aTileMap(XX + YY * \paRoom(iID)\Size\iWidth)
			 						
			 					
			 						
			 						If Lock = -1 
			 							If *SubTile\iFloor = *Tile\iFloor And *SubTile\fHeight = *Tile\fHeight And TileStatusMap(XX, YY) = #False 
											Size\iWidth + 1 
										Else
											Lock = XX
											Break
										EndIf
										
									ElseIf Lock > -1
										If *SubTile\iFloor <> *Tile\iFloor Or *SubTile\fHeight <> *Tile\fHeight Or TileStatusMap(XX, YY) = #True
		 									Size\iHeight - 1 
		 									Break 2
		 								EndIf
	
									EndIf
									
									If XX = \paRoom(iID)\Size\iWidth-1
			 							Lock = \paRoom(iID)\Size\iWidth
			 							Break
			 						EndIf
									
									If YY = \paRoom(iID)\Size\iHeight-1
										Break 2
									EndIf
									
								Next
								
						
								
								Size\iHeight + 1 
							Next
						
						
						Vertex = UBE_AddMeshData(\paRoom(iID)\lObject()\iMesh, Rooms\pCeilingMesh, X-\paRoom(iID)\Size\iWidth/2, *Tile\fHeight * 2.0, Y-\paRoom(iID)\Size\iHeight/2, 0,0,0, Size\iWidth, 1.0,  Size\iHeight)
							
						For I = 0 To 5
							Protected *TexCoord.UBE_VECTOR2f = UBE_GetMeshTexCoord(\paRoom(iID)\lObject()\iMesh, Vertex+I)
							*TexCoord\U * Size\iWidth
							*TexCoord\V * Size\iHeight
						Next
						
	
						For YY = Y To Y + Size\iHeight-1
						For XX = X To X + Size\iWidth-1
							TileStatusMap(XX, YY) = #True
						Next
						Next

					EndIf
				EndIf
				
				TileStatusMap(X, Y) = #True
				
			Next
			Next

			UBE_FinishMesh(\paRoom(iID)\lObject()\iMesh)
			UBE_CalculateMeshTangents(\paRoom(iID)\lObject()\iMesh)	
	
			\paRoom(iID)\lObject()\iObject 	= UBE_CreateObject(#UBE_ANY, \paRoom(iID)\lObject()\iMesh, \pCeilingMaterial)
																				UBE_MoveObject(\paRoom(iID)\lObject()\iObject, \lRoom()\Position\X, 0, \lRoom()\Position\Z)																 
																				UBE_SetObjectBody(\paRoom(iID)\lObject()\iObject, #UBE_COLLISION_BODY_MESH)

	EndWith
	
EndProcedure
									 
Procedure.i GenerateWall( iID.i, iMesh.i, iXShift.i, iYShift.i )
	
	Protected Dim TileStatusMap.i(0, 0)
	Protected I.i, X.i, Y.i, XX.i, YY.i
	Protected Size.UBE_SIZE2i
	Protected *Mesh.UBE_MESH = UBE_MeshID(iMesh)
	Protected *Tile.ROOM_TILE
	Protected	*SubTile.ROOM_TILE
	Protected Vertex.i
	Protected Lock.i
	Protected XShift.i , YShift.i
		Protected *TexCoord.UBE_VECTOR2f
	
	With Rooms	 
		
		Dim TileStatusMap.i(\paRoom(iID)\Size\iWidth, \paRoom(iID)\Size\iHeight)
	
		For Y = 0 To \paRoom(iID)\Size\iHeight-1
		For X = 0 To \paRoom(iID)\Size\iWidth-1
					
			If TileStatusMap(X, Y) = #False
				
				*Tile = \paRoom(iID)\aTileMap(X + Y * \paRoom(iID)\Size\iWidth)
				
					If X = 0 And iXShift < 0
						XShift = 0
					ElseIf X = \paRoom(iID)\Size\iWidth-1 And iXShift > 0
						XShift = 0
					Else
						XShift = iXShift
					EndIf
				
					If Y = 0 And iYShift < 0
						YShift = 0
					ElseIf Y = \paRoom(iID)\Size\iHeight-1 And iYShift > 0
						YShift = 0
					Else
						YShift = iYShift
					EndIf
					
					
					If Not (iXShift = - 1 And \paRoom(iID)\aDataMap(X + Y * \paRoom(iID)\Size\iWidth) = 4) And Not (iXShift = 1 And \paRoom(iID)\aDataMap(X + Y * \paRoom(iID)\Size\iWidth) = 2)
								
					If *Tile\iFloor > 0 And  ((\paRoom(iID)\aTileMap((X+XShift) + (Y+YShift) * \paRoom(iID)\Size\iWidth)\iFloor = 0 Or *Tile\fLevel < \paRoom(iID)\aTileMap((X+XShift) + (Y+YShift) * \paRoom(iID)\Size\iWidth)\fLevel Or *Tile\fHeight < \paRoom(iID)\aTileMap((X+XShift) + (Y+YShift) * \paRoom(iID)\Size\iWidth)\fHeight) Or (XShift = 0 And YShift = 0))
						
						Size\iWidth 	= 1
						Size\iHeight 	= 1
						
						
					
						
						
						
						If iYShift <> 0
							
							For XX = X+1 To \paRoom(iID)\Size\iWidth-1
								*SubTile = \paRoom(iID)\aTileMap(XX + Y * \paRoom(iID)\Size\iWidth)
									
								If *SubTile\iFloor > 0 And *SubTile\fLevel = *Tile\fLevel And *SubTile\fHeight = *Tile\fHeight And (\paRoom(iID)\aTileMap(XX + (Y+YShift) * \paRoom(iID)\Size\iWidth)\iFloor = 0  Or YShift = 0)
									
									
								
									Size\iWidth +1			
									
									
								Else
									Break	
								EndIf
									
							Next
							
							
							If *Tile\fHeight < \paRoom(iID)\aTileMap((X+XShift) + (Y+YShift) * \paRoom(iID)\Size\iWidth)\fHeight
								Protected TilefHeight.f = \paRoom(iID)\aTileMap((X+XShift) + (Y+YShift) * \paRoom(iID)\Size\iWidth)\fHeight - *Tile\fHeight
									
								If iYShift = -1
									Vertex = UBE_AddMeshData(*Mesh, Rooms\pWallMesh, X-\paRoom(iID)\Size\iWidth/2+1, (*Tile\fLevel+*Tile\fHeight) * 2.0, Y-\paRoom(iID)\Size\iHeight/2, 0,180,0, Size\iWidth, TilefHeight-*Tile\fLevel,  Size\iHeight)
								Else
									Vertex = UBE_AddMeshData(*Mesh, Rooms\pWallMesh, X-\paRoom(iID)\Size\iWidth/2+Size\iWidth-1, (*Tile\fLevel+*Tile\fHeight) * 2.0, Y-\paRoom(iID)\Size\iHeight/2+1, 0,0,0, Size\iWidth, TilefHeight-*Tile\fLevel,  Size\iHeight)
								EndIf
									
								For I = 0 To 5
									*TexCoord = UBE_GetMeshTexCoord(*Mesh, Vertex+I)
									*TexCoord\U * Size\iWidth
									If *TexCoord\V = 0
										*TexCoord\V-*Tile\fLevel
									Else
										*TexCoord\V * TilefHeight
									EndIf
								Next
									
							Else	
								If iYShift = -1
									Vertex = UBE_AddMeshData(*Mesh, Rooms\pWallMesh, X-\paRoom(iID)\Size\iWidth/2, *Tile\fLevel * 2.0, Y-\paRoom(iID)\Size\iHeight/2, 0,0,0, Size\iWidth, *Tile\fHeight-*Tile\fLevel,  Size\iHeight)
								Else
									Vertex = UBE_AddMeshData(*Mesh, Rooms\pWallMesh, X-\paRoom(iID)\Size\iWidth/2+Size\iWidth, *Tile\fLevel * 2.0, Y-\paRoom(iID)\Size\iHeight/2+1, 0,180,0, Size\iWidth, *Tile\fHeight-*Tile\fLevel,  Size\iHeight)
								EndIf
								
								
								For I = 0 To 5
									*TexCoord = UBE_GetMeshTexCoord(*Mesh, Vertex+I)
									*TexCoord\U * Size\iWidth
									If *TexCoord\V = 0
										*TexCoord\V-*Tile\fLevel
									Else
										*TexCoord\V * *Tile\fHeight
									EndIf
								Next
									
							EndIf
							
							
							
							
							
							
							
							
							
							
							
							
						ElseIf iXShift <> 0
						
							For YY = Y+1 To \paRoom(iID)\Size\iHeight-1
								*SubTile = \paRoom(iID)\aTileMap(X + YY * \paRoom(iID)\Size\iWidth)
									
								If *SubTile\iFloor > 0 And *SubTile\fLevel = *Tile\fLevel And *SubTile\fHeight = *Tile\fHeight And (\paRoom(iID)\aTileMap((X+XShift) + YY * \paRoom(iID)\Size\iWidth)\iFloor = 0  Or XShift = 0) And *SubTile\fLevel = \paRoom(iID)\aTileMap((X+XShift) + YY * \paRoom(iID)\Size\iWidth)\fLevel And *SubTile\fHeight = \paRoom(iID)\aTileMap((X+XShift) + YY * \paRoom(iID)\Size\iWidth)\fHeight
									
									If (iXShift = - 1 And \paRoom(iID)\aDataMap(X + YY * \paRoom(iID)\Size\iWidth) = 4) Or (iXShift = 1 And \paRoom(iID)\aDataMap(X + YY * \paRoom(iID)\Size\iWidth) = 2)
										Break
									EndIf
									
									Size\iHeight +1
									
									
									
								Else
									Break	
								EndIf
									
							Next
							
							
							If *Tile\fHeight < \paRoom(iID)\aTileMap((X+XShift) + (Y+YShift) * \paRoom(iID)\Size\iWidth)\fHeight
								TilefHeight.f = \paRoom(iID)\aTileMap((X+XShift) + (Y+YShift) * \paRoom(iID)\Size\iWidth)\fHeight - *Tile\fHeight
									
								If iXShift = -1
									Vertex = UBE_AddMeshData(*Mesh, Rooms\pWallMesh, X-\paRoom(iID)\Size\iWidth/2, (*Tile\fLevel+*Tile\fHeight) * 2.0, Y-\paRoom(iID)\Size\iHeight/2, 0,270,0, Size\iHeight, TilefHeight-*Tile\fLevel,  Size\iWidth)
								Else
									Vertex = UBE_AddMeshData(*Mesh, Rooms\pWallMesh, X-\paRoom(iID)\Size\iWidth/2+1, (*Tile\fLevel+*Tile\fHeight) * 2.0, Y-\paRoom(iID)\Size\iHeight/2+Size\iHeight, 0,90,0, Size\iHeight, TilefHeight-*Tile\fLevel,  Size\iWidth)
								EndIf
									
								For I = 0 To 5
									*TexCoord = UBE_GetMeshTexCoord(*Mesh, Vertex+I)
									*TexCoord\U * Size\iHeight
									If *TexCoord\V = 0
										*TexCoord\V-*Tile\fLevel
									Else
										*TexCoord\V * TilefHeight
									EndIf
								Next
									
							Else	
								If iXShift = -1
									Vertex = UBE_AddMeshData(*Mesh, Rooms\pWallMesh, X-\paRoom(iID)\Size\iWidth/2, *Tile\fLevel * 2.0, Y-\paRoom(iID)\Size\iHeight/2+Size\iHeight, 0,90,0, Size\iHeight, *Tile\fHeight-*Tile\fLevel,  Size\iWidth)
								Else
									Vertex = UBE_AddMeshData(*Mesh, Rooms\pWallMesh, X-\paRoom(iID)\Size\iWidth/2+1, *Tile\fLevel * 2.0, Y-\paRoom(iID)\Size\iHeight/2, 0,270,0, Size\iHeight, *Tile\fHeight-*Tile\fLevel,  Size\iWidth)
								EndIf
								
								For I = 0 To 5
									*TexCoord = UBE_GetMeshTexCoord(*Mesh, Vertex+I)
									*TexCoord\U * Size\iHeight
									If *TexCoord\V = 0
										*TexCoord\V-*Tile\fLevel
									Else
										*TexCoord\V * *Tile\fHeight
									EndIf
								Next
									
							EndIf
								
							
								
							
								
								
							
							
							
							
							
							
							
							
							
							
						
						
						EndIf
						
						
						For YY = Y To Y + Size\iHeight-1
						For XX = X To X + Size\iWidth-1
							TileStatusMap(XX, YY) = #True
						Next
						Next

					EndIf
					
					EndIf
					
				EndIf
				
				TileStatusMap(X, Y) = #True
				
			Next
			Next
			
	EndWith
	
			
EndProcedure

Procedure.i CreateRoom( iID.i, File.s, X.f = 0.0, Z.f = 0.0 )

	Protected FileID.i = ReadFile(#PB_Any, File)
	Protected String.s, FieldString.s, TileDataString.s
	Protected I.i, II.i, IX.i, IY.i, IIX.i, IIY.i
	Protected Dim TileStatusMap.i(0, 0)
	Protected Size.UBE_SIZE2i
	Protected *Tile.ROOM_TILE, *SubTile.ROOM_TILE
	Protected Lock.i
	Protected Vertex.i
	Protected *TexCoord.UBE_VECTOR2f
	Protected NewList lWindows.ROOM_WINDOW()
	Protected NewList lDoors.ROOM_DOOR()

	If FileID
		
		With Rooms
			
			AddElement(\lRoom())
			
			If ArraySize(\paRoom()) < iID
				ReDim \paRoom(iID)
			EndIf
			
			\paRoom(iID) 				= \lRoom()
			
			\lRoom()\iID 				= iID
			\lRoom()\Position\X = X
			\lRoom()\Position\Z = Z

			While Not Eof(FileID)
				String = Trim(ReadString(FileID))
				
				Select StringField(String, 1, ":")
						
					Case "Size"
						FieldString = StringField(String, 2, ":")
						\lRoom()\Size\iWidth = Val(StringField(FieldString, 1, "x"))
						\lRoom()\Size\iHeight = Val(StringField(FieldString, 2, "x"))
						ReDim	\lRoom()\aTileMap(\lRoom()\Size\iWidth * \lRoom()\Size\iHeight)
						ReDim \lRoom()\aDataMap(\lRoom()\Size\iWidth * \lRoom()\Size\iHeight)
						
					Case "Data"
						FieldString = StringField(String, 2, ":")
						For IX = 1 To \lRoom()\Size\iWidth
							TileDataString =  Trim(StringField(FieldString, IX, ","))
							\lRoom()\aTileMap(I)\iFloor 	= Val(StringField(TileDataString, 1, "/"))
							\lRoom()\aTileMap(I)\fLevel 	= ValF(StringField(TileDataString, 2, "/")) * 0.25
							\lRoom()\aTileMap(I)\fHeight 	= ValF(StringField(TileDataString, 3, "/"))
							I + 1
						Next
						
					Case "Data2"
						FieldString = StringField(String, 2, ":")
						For IX = 1 To \lRoom()\Size\iWidth
							\lRoom()\aDataMap(II) = Val(StringField(FieldString, IX, ","))
							II + 1
						Next
						
					Case "Window"
						FieldString = StringField(String, 2, ":")
						AddElement(lWindows())
						lWindows()\X 					= Val(StringField(FieldString, 1, ","))
						lWindows()\Y 					= Val(StringField(FieldString, 2, ","))
						lWindows()\fRotation 	= Val(StringField(FieldString, 3, ","))
						
					Case "Door"
						FieldString = StringField(String, 2, ":")
						AddElement(\lRoom()\lDoor())
						\lRoom()\lDoor()\X  					= Val(StringField(FieldString, 1, ","))
						\lRoom()\lDoor()\Z  					= Val(StringField(FieldString, 2, ","))
						\lRoom()\lDoor()\fRotation		= Val(StringField(FieldString, 3, ","))

				EndSelect

			Wend
			
			
			GenerateFloor(iID)		
			
			GenerateCeiling(iID)					
																			
			AddElement(\lRoom()\lObject())
			\lRoom()\lObject()\sName = "Wall"
			\lRoom()\lObject()\iMesh = UBE_CreateMesh(#UBE_ANY)		
			GenerateWall(iID, \lRoom()\lObject()\iMesh,  0, -1)
			GenerateWall(iID, \lRoom()\lObject()\iMesh,  0,  1)
			GenerateWall(iID, \lRoom()\lObject()\iMesh, -1,  0)
			GenerateWall(iID, \lRoom()\lObject()\iMesh,  1,  0)
			UBE_FinishMesh(\lRoom()\lObject()\iMesh)
			UBE_CalculateMeshTangents(\lRoom()\lObject()\iMesh)	
			\lRoom()\lObject()\iObject 	= UBE_CreateObject(#UBE_ANY, \lRoom()\lObject()\iMesh, \pWallMaterial)
																		UBE_MoveObject(\lRoom()\lObject()\iObject, \lRoom()\Position\X, 0, \lRoom()\Position\Z)																 
																		UBE_SetObjectBody(\lRoom()\lObject()\iObject, #UBE_COLLISION_BODY_MESH)
																													
			If ListSize(lWindows()) > 0											
				AddElement(\lRoom()\lObject())
				\lRoom()\lObject()\sName 				= "Window"
				\lRoom()\lObject()\iMesh 				= UBE_CreateMesh(#UBE_ANY)		
				ForEach lWindows()
					UBE_AddMeshData(\lRoom()\lObject()\iMesh, Rooms\pWindowMesh, lWindows()\X-\lRoom()\Size\iWidth/2, 0, lWindows()\Y-\lRoom()\Size\iHeight/2+0.5, 0,lWindows()\fRotation,0)
				Next
				UBE_FinishMesh(\lRoom()\lObject()\iMesh)
				UBE_CalculateMeshTangents(\lRoom()\lObject()\iMesh)	
				\lRoom()\lObject()\iObject 			= UBE_CreateObject(#UBE_ANY, \lRoom()\lObject()\iMesh, \pWindowMaterial)
																					UBE_MoveObject(\lRoom()\lObject()\iObject, \lRoom()\Position\X, 0, \lRoom()\Position\Z)																 
																					UBE_SetObjectBody(\lRoom()\lObject()\iObject, #UBE_COLLISION_BODY_MESH)
																												
				AddElement(\lRoom()\lObject())
				\lRoom()\lObject()\sName				= "Window Glass"
				\lRoom()\lObject()\iMesh 				= UBE_CreateMesh(#UBE_ANY)		
				ForEach lWindows()
					UBE_AddMeshData(\lRoom()\lObject()\iMesh, Rooms\pWindowGlassMesh, lWindows()\X-\lRoom()\Size\iWidth/2, 0, lWindows()\Y-\lRoom()\Size\iHeight/2+0.5, 0,lWindows()\fRotation,0)
				Next
				UBE_FinishMesh(\lRoom()\lObject()\iMesh)
				\lRoom()\lObject()\iObject 			= UBE_CreateObject(#UBE_ANY, \lRoom()\lObject()\iMesh, \pWindowGlassMaterial)
																					UBE_MoveObject(\lRoom()\lObject()\iObject, \lRoom()\Position\X, 0, \lRoom()\Position\Z)																 
				
				ForEach lWindows()
					AddElement(\lRoom()\lLight())
					If lWindows()\fRotation = 90
						\lRoom()\lLight()				  = UBE_CreateLight(#UBE_ANY, lWindows()\X-\lRoom()\Size\iWidth/2+0.25, 1.0, lWindows()\Y-\lRoom()\Size\iHeight/2)
					ElseIf lWindows()\fRotation = 270
						\lRoom()\lLight()				  = UBE_CreateLight(#UBE_ANY, lWindows()\X-\lRoom()\Size\iWidth/2-0.25, 1.0, lWindows()\Y-\lRoom()\Size\iHeight/2)
					EndIf		
																			UBE_SetLightColor(\lRoom()\lLight(), $FFFFAF97) 																	
																			UBE_SetLightRange(\lRoom()\lLight(), 4.0)
		 																	UBE_SetLightIntensity(\lRoom()\lLight(), 1.0)	
																			UBE_Attachment(\lRoom()\iObject, \lRoom()\lLight())													
					Next
				EndIf
				
				
				ForEach \lRoom()\lDoor()
					\lRoom()\lDoor()\X 				= X + \lRoom()\lDoor()\X - \lRoom()\Size\iWidth/2 -0.5
					\lRoom()\lDoor()\Z 				= Z + \lRoom()\lDoor()\Z - \lRoom()\Size\iHeight/2
					\lRoom()\lDoor()\iObject 	= UBE_CreateObject(#UBE_ANY, \pDoorMesh, \pDoorMaterial)				
																			UBE_MoveObject(\lRoom()\lDoor()\iObject, \lRoom()\lDoor()\X, 0.0, \lRoom()\lDoor()\Z)																 
																			UBE_RotateObject(\lRoom()\lDoor()\iObject, 0.0, \lRoom()\lDoor()\fRotation, 0.0)
																			UBE_SetObjectBody(\lRoom()\lDoor()\iObject, #UBE_COLLISION_BODY_BOX)
					
					AddElement(\lRoom()\lDoor()\lLight())
					\lRoom()\lDoor()\lLight() = UBE_CreateLight(#UBE_ANY, 0.2, 1.8, -0.5)
																			UBE_SetLightColor(\lRoom()\lDoor()\lLight(), $FFFFAF97)
																			UBE_SetLightRange(\lRoom()\lDoor()\lLight(), 1.25)
																			UBE_SetLightIntensity(\lRoom()\lDoor()\lLight(), 0.6)														
																			UBE_Attachment(\lRoom()\lDoor()\iObject, \lRoom()\lDoor()\lLight())	
				Next

		EndWith
		
		CloseFile(FileID)
		ProcedureReturn #True
	EndIf
	
	ProcedureReturn #False	
	
EndProcedure

	
	
; IDE Options = PureBasic 5.21 LTS (Windows - x86)
; CursorPosition = 691
; FirstLine = 202
; Folding = 5
; EnableXP